from validation.value_validation import ValidateValue

def bmr_results(client):
    results = BMRFormulas(client).add_bmr_results_to_dictionary()
    return results

def harris_benedict_bmr(client):
    results = BMRFormulas(client).add_bmr_results_to_dictionary()
    harris_benedict = results['harris_benedict_bmr']
    return harris_benedict

def mifflin_bmr(client):
    results = BMRFormulas(client).add_bmr_results_to_dictionary()
    mifflin = results['mifflin_st_jeor_bmr']
    return mifflin


class MetabolicClient(object):
    def __init__(self, sex, weight, height, age):
        self._sex = ValidateValue(sex).valueIsMaleFemale()
        self._weight = ValidateValue(weight).valueIsPositiveFloat()
        self._height = ValidateValue(height).valueIsPositiveFloat()
        self._age = ValidateValue(age).valueIsPositiveInteger()

class BMRFormulas(object):
    def __init__(self, client):
        self._client = client


    def add_bmr_results_to_dictionary(self):
        formulas = {
            'harris_benedict_bmr':self._harris_benedict_bmr(),
            'mifflin_st_jeor_bmr':self._mifflin_st_jeor_bmr()
        }
        results = {}
        for key, value in formulas.items():
            results[key] = value

        return results


    def _harris_benedict_bmr(self):
        if self._client._sex == 'male':
            bmr = (88.4 + 13.4 * self._client._weight) + (4.8 * self._client._height) - (5.68 * self._client._age)

        if self._client._sex == 'female':
            bmr = (447.6 + 9.25 * self._client._weight) + (3.10 * self._client._height) - (4.33 * self._client._age)

        return bmr


    def _mifflin_st_jeor_bmr(self):
        if self._client._sex == 'male':
            bmr = (9.99 * self._client._weight) + (6.25 * self._client._height) - (4.92 * self._client._age) + 5

        if self._client._sex == 'female':
            bmr = (9.99 * self._client._weight) + (6.25 * self._client._height) - (4.92 * self._client._age) - 161

        return bmr



from .metabolic_rate_prediction import bmr_results


def get_calories_by_activity(activity_level, bmr):
    ACTIVITY_FACTORS = {
        'sedentary': 1.2,
        'lightly active': 1.375,
        'moderately active': 1.55,
        'very active': 1.725,
        'extremely active': 1.9
    }

    if activity_level in ACTIVITY_FACTORS:
        results = ACTIVITY_FACTORS[activity_level] * bmr
        return results



class ValidateValue():
    _female_male = ['female', 'male']
    _yes_no = {
                'yes': True,
                'no': False
                }

    def __init__(self, value):
        self._value = value

    def valueIsMaleFemale(self):
        if self._value in ValidateValue._female_male:
            return self._value
        else:
            raise ValueError('%s is an incorrect value. It must be male or female.' % self._value)

    def valueIsPositiveInteger(self):
        integer_value = int(self._value)
        if integer_value >= 0:
            return integer_value
        else:
            raise ValueError('%s is an incorrect value. It must be a number and must higher than zero.' % self._value)

    def valueIsPositiveFloat(self):
        float_value = float(self._value)
        if float_value >= 0:
            return float_value
        else:
            raise ValueError('%s is an incorrect value. It must be a number and must higher than zero.' % self._value)

    def valueIsYesNo(self):
        if self._value in ValidateValue._yes_no.keys():
            return ValidateValue._yes_no[self._value]
        else:
            raise ValueError('%s is an incorrect value. Must be yes or no.' % self._value)

    def valueIsLessThan10(self):
        if ValidateValue(self._value).valueIsPositiveInteger() <= 10:
            return self._value
        else:
            raise ValueError('%s is an incorrect value.  Must be 10 or less.' % self._value)



import math


from validation.value_validation import ValidateValue


def one_rep_max_estimation(reps, weight):
    results = OneRepMaxEquations(reps,weight).add_results_to_dictionary()
    return results


class OneRepMaxEquations(object):
    def __init__(self, reps, weight):
        self._reps = float(ValidateValue(reps).valueIsLessThan10())
        self._weight = ValidateValue(weight).valueIsPositiveFloat()

    def add_results_to_dictionary(self):
        formulas = {
            'brzycki_formula': self._brzycki_formula(),
            'epley_formula': self._epley_formula(),
            'mayhew_formula': self._mayhew_formula(),
            'lander_formula': self._lander_formula(),
            'lombardi_formula': self._lombardi_formula()
        }

        results = {}

        for key, value in formulas.items():
            results[key] = value

        return results

    def _brzycki_formula(self):
        brzycki_one_rep_max = self._weight/(1.0278-(0.0278*self._reps))
        return brzycki_one_rep_max

    def _epley_formula(self):
        # Reps need to be more than 1
        epley_one_rep_max = self._weight * (1 + (self._reps/30))
        return epley_one_rep_max

    def _mayhew_formula(self):
        mayhew_one_rep_max = (100 * self._weight)/(52.2 + (41.9 * math.exp(-0.055**self._reps)))
        return mayhew_one_rep_max

    def _lander_formula(self):
        lander_one_rep_max = self._weight/(1.013-(0.0267123*self._reps))
        return lander_one_rep_max

    def _lombardi_formula(self):
        lombardi_one_rep_max = self._weight*self._reps**0.10
        return lombardi_one_rep_max



from validation.value_validation import ValidateValue

def risk_classification(patient):
    classification = RiskFactorAssessment(patient).result_risk_classification()
    return classification

def total_risk_factors(patient):
    total = RiskFactorAssessment(patient).net_risk_factors()
    return total

class Patient(object):
    def __init__(self, sex, age, smoker, sedentary, bmi, waist_girth, male_family_death_before_55, female_family_death_before_65, systolic, diastolic, hypertensive, ldl, hdl, using_lipid_lowering_medication, cholesterol, fasting_glucose, oral_glucose_tolerance):
        self._sex = ValidateValue(sex).valueIsMaleFemale()
        self._age = ValidateValue(age).valueIsPositiveInteger()
        self._smoker = ValidateValue(smoker).valueIsYesNo()
        self._sedentary = ValidateValue(sedentary).valueIsYesNo()
        self._bmi = ValidateValue(bmi).valueIsPositiveInteger()
        self._waist_girth = ValidateValue(waist_girth).valueIsPositiveInteger()
        self._male_family_death_before_55 = ValidateValue(male_family_death_before_55).valueIsYesNo()
        self._female_family_death_before_65 = ValidateValue(female_family_death_before_65).valueIsYesNo()
        self._systolic = ValidateValue(systolic).valueIsPositiveInteger()
        self._diastolic = ValidateValue(diastolic).valueIsPositiveInteger()
        self._hypertensive = ValidateValue(hypertensive).valueIsYesNo()
        self._ldl = ValidateValue(ldl).valueIsPositiveInteger()
        self._hdl = ValidateValue(hdl).valueIsPositiveInteger()
        self._using_lipid_lowering_medication = ValidateValue(using_lipid_lowering_medication).valueIsYesNo()
        self._cholesterol = ValidateValue(cholesterol).valueIsPositiveInteger()
        self._fasting_glucose = ValidateValue(fasting_glucose).valueIsPositiveInteger()
        self._oral_glucose_tolerance = ValidateValue(oral_glucose_tolerance).valueIsPositiveInteger()


class RiskFactorAssessment(object):
    def __init__(self, patient):
        self._patient = patient

    def result_risk_classification(self):
        classification = RiskFactorAssessmentClassification().patient_risk_classification(self.net_risk_factors())
        return classification

    def net_risk_factors(self):
        total = self._get_risk_factor_count() - self._get_negative_risk_factor_count()
        return total

    def _get_risk_factor_count(self):
        _count_risk_factors = [
            self._is_age_risk(),
            self._is_obesity_risk(),
            self._patient._smoker,
            self._patient._sedentary,
            self._is_familial_risk(),
            self._is_systolic_risk(),
            self._is_diastolic_risk(),
            self._patient._hypertensive,
            self._is_dyslipidemia_risk(),
            self._is_pre_diabetes_risk()
            ]
        return _count_risk_factors.count(True)

    def _get_negative_risk_factor_count(self):
        _count_negative_risk_factor = [ self._is_hdl_negative_risk() ]
        return _count_negative_risk_factor.count(True)

    def _is_obesity_risk(self):
        if self._patient._bmi and self._patient._waist_girth == 0:
            return False
        elif self._patient._bmi > 30:
            return True
        elif self._patient._waist_girth > 40 and self._patient._sex == 'male' or self._patient._waist_girth > 35 and self._patient._sex == 'female':
            return True
        else:
            return False

    def _is_age_risk(self):
        if (self._patient._sex == "male" and self._patient._age >= 45) or (self._patient._sex == "female" and self._patient._age >=55):
            return True
        else:
            return False

    def _is_familial_risk(self):
        if self._patient._male_family_death_before_55 == True or self._patient._female_family_death_before_65 == True:
            return True
        else:
            return False

    def _is_systolic_risk(self):
        if self._patient._systolic >= 120:
            return True
        else:
            return False

    def _is_diastolic_risk(self):
        if self._patient._diastolic >= 80:
            return True
        else:
            return False

    def _is_dyslipidemia_risk(self):
        if self._patient._ldl > 130 or self._patient._hdl < 40 or self._patient._cholesterol > 200:
            return True
        else:
            return False

    def _is_pre_diabetes_risk(self):
        if self._patient._fasting_glucose >= 100 and self._patient._fasting_glucose <= 126 or self._patient._oral_glucose_tolerance >= 140 and self._patient._oral_glucose_tolerance < 200:
            return True
        else:
            return False

    def _is_hdl_negative_risk(self):
        if self._patient._hdl > 60:
            return True
        else:
            return False

class RiskFactorAssessmentClassification(object):
    def __init__(self):
        pass

    def patient_risk_classification(self, value):
        if value <= 1:
            return self._low_risk_category(value)

        if value == 2:
            return self._moderate_risk_category(value)

        if value >  2:
            return self._high_risk_category(value)

    def _low_risk_category(self, value):
        return("Your risk total is %s. You are at a low risk for cardiovascular disease. Medical check-up no necessary for participation in physical activity." % value)

    def _moderate_risk_category(self, value):
        return("Your risk total is %s. You are at a moderate risk for cardiovascular disease. Medical check-up recommended for participation in vigorous physical activity." % value)

    def _high_risk_category(self, value):
        return("Your risk total is %s. You are at a high risk for cardiovascular disease. Medical check-up highly recommended before any physical activity." % value)



from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='exercisecoachtools',
    version='0.1.1',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    python_requires='>=3.7, <4',
    url='https://github.com/chandojo/ExerciseCoachTools',
    license='MIT License',
    author='chandojo',
    description='Tools for exercise physiologists and coaches.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    setup_requires=['pytest-runner'],
    tests_require=['pytest']
)
